@@ -92,6 +92,7 @@ elements_matroska = (
                 'MaxBlockAdditionID, 55ee, uint',
                 'Name, 536e, str',
                 'Language, 22b59c, str',
+                'LanguageBCP47, 22b59d, str',
                 'CodecID, 86, str',
                 'CodecPrivate, 63a2, binary',
                 'CodecName, 258688, str',
@@ -105,6 +106,10 @@ elements_matroska = (
                     'DisplayWidth, 54b0, uint',
                     'DisplayHeight, 54ba, uint',
                     'DisplayUnit, 54b2, uint',
+                    'PixelCropTop, 54bb, uint',
+                    'PixelCropLeft, 54cc, uint',
+                    'PixelCropRight, 54dd, uint',
+                    'PixelCropBottom, 54aa, uint',
                     'FrameRate, 2383e3, float',
                     'ColourSpace, 2eb524, binary',
                     'StereoMode, 53b8, uint',
@@ -160,6 +165,12 @@ elements_matroska = (
                         ),
                     ),
                 ),
+                'BlockAdditionMapping*, 41e4, sub', (
+                    'BlockAddIDValue, 41f0, uint',
+                    'BlockAddIDName, 41a4, str',
+                    'BlockAddIDType, 41e7, uint',
+                    'BlockAddIDExtraData, 41ed, binary',
+                ),
             ),
         ),
 
@@ -202,6 +213,7 @@ elements_matroska = (
                     'ChapterDisplay*, 80, sub', (
                         'ChapString, 85, str',
                         'ChapLanguage*, 437c, str',
+                        'ChapLanguageBCP47*, 437d, str',
                         'ChapCountry*, 437e, str',
                     ),
                 ),
@@ -220,6 +232,7 @@ elements_matroska = (
                 'SimpleTag*, 67c8, sub', (
                     'TagName, 45a3, str',
                     'TagLanguage, 447a, str',
+                    'TagLanguageBCP47, 447b, str',
                     'TagString, 4487, str',
                     'TagDefault, 4484, uint',
                 ),
@@ -459,15 +472,10 @@ if __name__ == "__main__":
     def parse_toplevel(s):
         parse_one(s, 0, None, 1 << 63)
 
-    if len(sys.argv) < 3:
-        outfile = sys.stdout
-    else:
-        outfile = open(sys.argv[2], "w")
-
     if sys.argv[1] == '--generate-header':
-        generate_C_header(outfile)
+        generate_C_header(open(sys.argv[2], "w"))
     elif sys.argv[1] == '--generate-definitions':
-        generate_C_definitions(outfile)
+        generate_C_definitions(open(sys.argv[2], "w"))
     else:
         s = open(sys.argv[1], "rb")
         while 1:
